/*
  Target tables specifically on the CLI Reference page.
  The .md-page--reference-clireference class is added by Material for MkDocs
  to the body tag of that specific page.
*/
.autowt-clitable-wrapper + div table th:first-of-type,
.autowt-clitable-wrapper + div table td:first-of-type {
  width: 25% !important;
  min-width: 10rem !important;
}